#ifndef GRIND__H
#define GRIND__H

#include <stdio.h>

#include "../graphics/graphics.h"

#define TINY 1.0e-12

#define LINE_LENGTH 1024
#define MAX_ARG 512
#define NAME_LENGTH 128

#define VARIABLE 0
#define EXPRESSION 1
#define PARAMETER 2

#define ROW4A 0
#define EULER 1
#define RK 2

#define true 1
#define false 0
#define gaussian 1
#define uniform  2

#define X_AXIS 0
#define Y_AXIS 1
#define Z_AXIS 2
#define T_AXIS 3
#define V_AXIS 4
#define H_AXIS 5

#define NONE  0
#define ALPHA 1
#define DIM2  2
#define DIM3  3

#define BOTH 0
#define UP   1
#define DOWN 2

#define NEUTRAL 2
#define STABLE -1
#define UNSTABLE 1

#define USE_GET_LINE 1

#define DEBUG 0

/*********************** TYPE DEFINITIONS ETC ************************/

typedef int boolean;

typedef struct {
  char *label;
  int var;
  double min;
  double max;
  int log;
  int ticks;
} _axis;

typedef struct {
  const char *name;
  double value;
  boolean init;
  int noise;
  double sd;
  boolean pos;
} _par;

typedef struct {
  const char *name;
  boolean init;
} _var;

typedef struct {
  const char *name;
  boolean init;
} _expr;

typedef struct {
  char *name;
  int (*funcptr)(int, char**);
} _symb_table;

extern int interrupt;
int Interrupt();

extern int pargc;
extern char **pargv;
extern int assign;

extern double *istate, *nstate;
extern double * const fstate;
extern double **rundata;
extern int rundatalength;
extern boolean *outputbuf;
extern const int nvar;
extern const int nexpr;
extern const int npar;
extern const int nnames;
extern double all_values[];
extern double *const par_values;
extern const char *all_names[];
extern int *all_types;
extern int positive_opt, forward_opt;
extern int jacob_opt;

/*  options.c */
extern boolean adjust_opt;
extern boolean attained_opt;
extern boolean axis_opt;
extern boolean backward_opt;
extern boolean calcul_opt;
extern boolean closed_opt;
extern boolean echo_opt;
extern boolean initial_opt;
extern boolean erase_opt;
extern boolean keepvar_opt;
extern boolean label_opt;
extern boolean logfile_opt;
extern boolean map_opt;
extern boolean mark_opt;
extern boolean points_opt;
extern boolean positive_opt;
extern boolean trunc_opt;

extern int mode_opt, mode_none;

/*  globals.c */
extern _axis axis[6];
extern _par *pars;
extern _var *vars;
extern _expr *exprs;
extern char dformat[];
extern char lformat[];
extern char vformat[];
extern char aformat[];
extern char sformat[];



/*  calc.c  */
double calc(char*);

/* commands.c */
int Assign(int, char**);
int Axis(int, char**);
int Bye(int, char**);
int Calc(int, char**);
int Bifurcate(int, char**);
int Continue(int, char**);
int Cursor(int, char**);
int Display(int, char**);
int E2n(int, char**);
int E2r(int, char**);
int E3n(int, char**);
int E3r(int, char**);
int Eigen(int, char**);
int Equilibrium(int,char**);
int Erase(int,char**);
int Ert(int, char**);
int Finish(int, char**);
int Format(int, char**);
int Graphics(int,char**);
int Grid(int,char**);
int Help(int, char**);
int Integrator(int, char**);
int Keepvar(int, char**);
int Load(int, char**);
int Macro(int, char**);
int Mode(int, char**);
int Nep(int, char**);
int Newton(int, char**);
int Noise(int,char**);
int Nullcline(int,char**);
int Option(int, char**);
int Output(int, char**);
int Parameter(int, char**);
int Perturb(int, char**);
int Plot(int, char**);
int Poincare(int, char**);
int Export(int, char**);
int Readfile(int, char**);
int GR_RGB(int, char**);
int Run(int, char**);
int Shade(int, char**);
int Takens(int, char**);
int ThreeD(int, char**);
int Timeplot(int, char**);
int Triangle(int, char**);
int TwoD(int, char**);
int Value(int, char**);
int Vector (int, char**);
int Where(int, char**);
int Exec(int, char**);

/*  contEqui.c */
int contEqui(int, double, int);

/* eigen.c */
int eigen(double*, boolean);
int prieig(double[], boolean);

/*  eispack.c */
int rg_(int*, int*, double*, double*, double*, int*, double*, int*, double*, int*);

/*  euler.c */
int euler(double*, int, double, double);

/* export.c  (either in gl/ or in X11/) */
void writepsfile(int, int, int, int, char *); /* in gl/ */
int requestPSdump(int, int, char *);          /* in gl/ */
int checkPSdump();                            /* in gl/ */
int export( int, char *[], char **);          /* in gl/ and in X11/ */

/*  functions.c */
int isign(double);
double d_sign(double*, double*);
double sign(double, double);
double heaviside(double);

/*  getCommand.c */
int ParseArgs();
int GetCommand(boolean);
int Initialize(int, char**);

/*  gldraw.c */
int drawcall(const _grcall**, const _grcall * const last_of_stack);

/* globals.c */
int GrindMainLoop(boolean);

/*  gps.c */
int gps(char*);

/*  graphics.c */
void startgraphics();
int closegraphics();
int checkgraphics();
int init();
void *initGL(void*);
int drawstack(const _grcall *, const _grcall *);
void display();
void resetView();
void setProjectionMatrix ();
int setMVmatrix();
void getrealcoord(int, int, double*, double*, double*);
void mouse(int, int, int, int);
void motion(int, int);
void keyboard(unsigned char, int, int);
void reshape(int, int);
void idle();

/*  grstate.c */
double val2ax(double, int);
double ax2val(double, int);
int displayPosition(double*);
int setMark(const double*, boolean);
boolean storeaxis(boolean);
int setax(double*, double, int);
int triangle(double*, double*);
int start_continuation(boolean);
int start_trajectory(int);
int add2trajectory();
int drawTimeSeries(boolean*);
int Draw2Daxis();
int Draw3Daxis();
int drawTimeAxis();
int DrawTakens(int);
boolean outside(double,double,double);

/*  integr.c */
int onestep(double, double, double[]);
int integr(FILE*);
int plot();

/*  misc.c */
int Index(char[], char[]);
int nameIndex(char*);
int eqf(char[], char[]);
int lowerword(char[], char[]);
int coparr(int, double[], double[]);
int prider();
int sort(int,double[],int[]);
char *storeStr(const char *);
char *storeString(char *, const char *);
double *dcalloc(int,char[]);
double **ddcalloc(int,char[]);
int *icalloc(int,char[]);
int **iicalloc(int,char[]);
char *ccalloc(int,char[]);
boolean *bcalloc(int,char[]);
int chckin(void);
int grepfile(FILE*, char*);
double znormal(double, double, boolean);
double zuniform();
boolean gr_warning(const char * const, ...);
int gHelp(char*, char*, char*);
int eqArg(int, char*, int, char**);
int abbrevArg(int, char*, int, char**);
int isnumeric(int, int, char**);
int mkVarPos(double[]);
int mkExprPos();

/*  model.c */
int mine(int, char*[]);
void fcn(double[], double[]);
void jac(double[], double[]);

/*  mouse2state.c */
int mouse2state(double*);

/*  mutex.c */
int readmousecoord(double*, double*, double*);
int writemousecoord(int, int, int, int);
int checkErase(_grcall*, _grcall**);
int Signal2ResetStack();
int append2stack(_grcall **, _grcall *);
int check4append(_grcall **, _grcall **);
int requestPSdump(int, int, char*);
int checkPSdump();
int LockStack();
int UnlockStack();

/*  nr: */
void odeint(double[], int, double, double, double, double,double, int*, int*,
	    void (*derivs)(double, double [], double []),
	    void (*rkqs)(double [], double [], int, double *, double, double, 
			 double [], double *, double *, 
			 void (*)(double, double [], double [])));
void rkqs(double [], double [], int, double *, double, double, double [],
	  double *, double *, void (*)(double, double [], double []));
int mnewt(int, double[], int, double, double);
float gasdev(long*);
float ran1(long*);

/* nrutil.c */
void nfcn(double, double [], double []);

/*  nullcline.c */
int shade(int, int, int);
int nullcline(int);

/*  perturb.c */
int perturb(int, double);

/*  poinc.c */

/*  row4a.c */
int row4a(int, double*, double*, double, double, double, double, int*, int*);

/*  user supplied: */
void fcn(double[], double[]);

/*  vectorfield.c */
int vectorfield(int);

/*  writepsfile.c */
void writepsfile(int, int, int, int, char*);

#endif
